@@ -680,9 +680,28 @@ end
 
 
 local bin_lookup = {
-  [0]="0000", "0001", "0010", "0011", "0100", "0101", "0110", "0111",
-      "1000", "1001", "1010", "1011", "1100", "1101", "1110", "1111",
+  ["0"]="0000",
+  ["1"]="0001",
+  ["2"]="0010",
+  ["3"]="0011",
+  ["4"]="0100",
+  ["5"]="0101",
+  ["6"]="0110",
+  ["7"]="0111",
+  ["8"]="1000",
+  ["9"]="1001",
+  ["a"]="1010",
+  ["b"]="1011",
+  ["c"]="1100",
+  ["d"]="1101",
+  ["e"]="1110",
+  ["f"]="1111",
 }
+setmetatable(bin_lookup, {
+    __index = function()
+      error("Error in ipOps.hex_to_bin: Expected string representing a hexadecimal number.")
+    end
+  })
 ---
 -- Converts a string of hexadecimal digits into the corresponding string of
 -- binary digits.
@@ -699,14 +718,7 @@ hex_to_bin = function( hex )
     return nil, "Error in ipOps.hex_to_bin: Expected string"
   end
 
-  local status, result = pcall( string.gsub, hex, ".", function(nibble)
-      local n = bin_lookup[tonumber(nibble, 16)]
-      if n then
-        return n
-      else
-        error("Error in ipOps.hex_to_bin: Expected string representing a hexadecimal number.")
-      end
-    end)
+  local status, result = pcall( string.gsub, string.lower(hex), ".", bin_lookup)
   if status then
     return result
   end
@@ -820,7 +832,7 @@ end
 do
   for _, h in ipairs({
       {"a", "1010"},
-      {"aa", "10101010"},
+      {"aA", "10101010"},
       {"12", "00010010"},
       {"54321", "01010100001100100001"},
       {"123error", false},
