@@ -43,7 +43,7 @@ end
 function login_user(socket, user, pw)
   socket:send("USER " .. user .. "\r\n")
   local status, line = socket:receive_lines(1)
-  if not stat(line) then return false, err.user_error end
+  if not stat(line) then return false, err.userError end
   socket:send("PASS " .. pw .. "\r\n")
 
   status, line = socket:receive_lines(1)
