@@ -32,7 +32,7 @@
 -- fetched from the registry (afaik) but not currently implemented. Some object ids are static : the registry is always 0
 --
 -- @author Martin Holst Swende
--- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
+-- @copyright Same as Nmap--See https://nmap.org/book/man-legal.html
 -- @see java 1.4 RMI-spec: http://java.sun.com/j2se/1.4.2/docs/guide/rmi/
 -- @see java 5 RMI-spec: http://java.sun.com/j2se/1.5.0/docs/guide/rmi/spec/rmiTOC.html
 -- @see java 6 RMI-spec : http://java.sun.com/javase/6/docs/technotes/guides/rmi/index.html
@@ -42,8 +42,6 @@
 -- Created 09/06/2010 - v0.1 - created by Martin Holst Swende <martin@swende.se>
 -- Fixed more documentation - v0.2 Martin Holst Swende <martin@swende.se>
 
-local bin = require "bin"
-local bit = require "bit"
 local nmap = require "nmap"
 local stdnse = require "stdnse"
 local string = require "string"
@@ -53,7 +51,7 @@ _ENV = stdnse.module("rmi", stdnse.seeall)
 
 local function dbg(str,...)
   local arg={...}
-  stdnse.print_debug(3,"RMI:"..str, table.unpack(arg))
+  stdnse.debug3("RMI:"..str, table.unpack(arg))
 end
 -- Convenience function to both print an error message and return <false, msg>
 -- Example usage :
@@ -62,7 +60,7 @@ end
 -- end
 local function doh(str,...)
   local arg={...}
-  stdnse.print_debug("RMI-ERR:"..tostring(str), table.unpack(arg))
+  stdnse.debug1("RMI-ERR:"..tostring(str), table.unpack(arg))
   return false, str
 end
 
@@ -70,18 +68,18 @@ end
 -- BufferedWriter
 -- This buffering writer provide functionality much like javas BufferedWriter.
 --
--- BufferedWriter wraps the pack-functionality from bin, and buffers data internally
+-- BufferedWriter wraps string.pack, and buffers data internally
 -- until flush is called. When flush is called, it either sends the data to the socket OR
 -- returns the data, if no socket has been set.
 --@usage:
 -- local bWriter = BufferedWriter:new(socket)
 -- local breader= BufferedReader:new(socket)
 --
--- bWriter.pack('>i', integer)
+-- bWriter.pack('>i4', integer)
 -- bWriter.flush() -- sends the data
 --
 -- if bsocket:canRead(4) then -- Waits until four bytes can be read
---   local packetLength = bsocket:unpack('i') -- Read the four bytess
+--   local packetLength = bsocket:unpack('>i4') -- Read the four bytess
 --   if bsocket:canRead(packetLength) then
 --     -- ...continue reading packet values
 
@@ -103,10 +101,10 @@ BufferedWriter = {
   send = function( self, data )
     self.writeBuffer = self.writeBuffer .. data
   end,
-  -- Convenience function, wraps bin
+  -- Convenience function, wraps string.pack
   pack = function(self, fmt, ... )
     local arg={...}
-    self.writeBuffer = self.writeBuffer .. bin.pack( fmt, table.unpack(arg))
+    self.writeBuffer = self.writeBuffer .. string.pack( fmt, table.unpack(arg))
   end,
 
   -- This function flushes the buffer contents, thereby emptying
@@ -133,7 +131,7 @@ BufferedWriter = {
 -- If not enough data is available, it blocks until data is received, thereby handling the case
 -- if data is spread over several tcp packets (which is a pitfall for many scripts)
 --
--- It wraps unpack from bin for the reading.
+-- It wraps string.unpack for the reading.
 -- OBS! You need to check before invoking skip or unpack that there is enough
 -- data to read. Since this class does not parse arguments to unpack, it does not
 -- know how much data to read ahead on those calls.
@@ -141,11 +139,11 @@ BufferedWriter = {
 -- local bWriter = BufferedWriter:new(socket)
 -- local breader= BufferedReader:new(socket)
 --
--- bWriter.pack('>i', integer)
+-- bWriter.pack('>i4', integer)
 -- bWriter.flush() -- sends the data
 --
 -- if bsocket:canRead(4) then -- Waits until four bytes can be read
---   local packetLength = bsocket:unpack('i') -- Read the four bytess
+--   local packetLength = bsocket:unpack('>i4') -- Read the four bytess
 --   if bsocket:canRead(packetLength) then
 --     -- ...continue reading packet values
 
@@ -191,27 +189,27 @@ BufferedReader = {
     return #self.readBuffer +1 -self.pos
   end,
   ---
-  -- This function works just like bin.unpack (in fact, it is
+  -- This function works just like string.unpack (in fact, it is
   -- merely a wrapper around it.  However, it uses the data
   -- already read into the buffer, and the internal position
-  --@param format - see bin
+  --@param format
   --@return the unpacked value (NOT the index)
   unpack = function(self,format)
-    local ret = {bin.unpack(format, self.readBuffer, self.pos)}
-    self.pos = ret[1]
-    return table.unpack(ret,2)
+    local ret = {string.unpack(format, self.readBuffer, self.pos)}
+    self.pos = ret[#ret]
+    ret[#ret] = nil
+    return table.unpack(ret)
   end,
   ---
-  -- This function works just like bin.unpack (in fact, it is
+  -- This function works just like string.unpack (in fact, it is
   -- merely a wrapper around it.  However, it uses the data
   -- already read into the buffer, and the internal position.
   -- This method does not update the current position, and the
   -- data can be read again
-  --@param format - see bin
+  --@param format
   --@return the unpacked value (NOT the index)
   peekUnpack = function(self,format)
-    local ret = {bin.unpack(format, self.readBuffer, self.pos)}
-    return table.unpack(ret,2)
+    return string.unpack(format, self.readBuffer, self.pos)
   end,
   ---
   -- Tries to read a byte, without consuming it.
@@ -219,7 +217,7 @@ BufferedReader = {
   --@return bytevalue
   peekByte = function(self)
     if self:canRead(1) then
-      return true, self:peekUnpack('C')
+      return true, self:peekUnpack('B')
     end
     return false
   end,
@@ -238,20 +236,20 @@ BufferedReader = {
 }
 
 -- The classes are generated when this file is loaded, by the definitions in the JavaTypes
--- table. That table contains mappings between the format used by bin and the types
--- available in java, aswell as the lengths (used for availability-checks) and the name which
+-- table. That table contains mappings between the format used by string.pack and the types
+-- available in java, as well as the lengths (used for availability-checks) and the name which
 -- is prefixed by read* or write* when monkey-patching the classes and adding functions.
 -- For example: {name = 'Int', expr = '>i', len=  4}, will generate the functions
 -- writeInt(self, value) and readInt() respectively
 
 local JavaTypes = {
-  {name = 'Int', expr = '>i', len=  4},
-  {name = 'UnsignedInt', expr = '>I', len=  4},
-  {name = 'Short', expr = '>s', len=  2},
-  {name = 'UnsignedShort', expr = '>S', len=  2},
-  {name = 'Long', expr = '>l', len=  8},
-  {name = 'UnsignedLong', expr = '>L', len=  8},
-  {name = 'Byte', expr = '>C', len=  1},
+  {name = 'Int', expr = '>i4', len=  4},
+  {name = 'UnsignedInt', expr = '>I4', len=  4},
+  {name = 'Short', expr = '>i2', len=  2},
+  {name = 'UnsignedShort', expr = '>I2', len=  2},
+  {name = 'Long', expr = '>i8', len=  8},
+  {name = 'UnsignedLong', expr = '>I8', len=  8},
+  {name = 'Byte', expr = '>B', len=  1},
 }
 
 ---
@@ -294,7 +292,7 @@ JavaDOS = {
 
   writeUTF = function(self, text)
     -- TODO: Make utf-8 of it
-    return self:pack('>P', text)
+    return self:pack('>s2', text)
   end,
   pack = function(self, ...)
     local arg={...}
@@ -356,8 +354,8 @@ JavaDIS = {
     if not self.bReader:canRead(2)  then-- Length of the string is two bytes
       return false, "Not enough data in buffer [0]"
     end
-    -- We do it as a 'peek', so bin can reuse the data to unpack with 'P'
-    local len = self.bReader:peekUnpack('>S')
+    -- We do it as a 'peek', so string.unpack can reuse the data to unpack with '>s2'
+    local len = self.bReader:peekUnpack('>I2')
     --dbg("Reading utf, len %d" , len)
     -- Check that we have data
     if not self.bReader:canRead(len) then
@@ -368,7 +366,7 @@ JavaDIS = {
     local val = self.bReader.readBuffer:sub(self.bReader.pos+2, self.bReader.pos+len+2-1)
     self.bReader.pos = self.bReader.pos+len+2
     -- Someone smarter than me can maybe get this working instead:
-    --local val = self.bReader:unpack('P')
+    --local val = self.bReader:unpack('>s2')
     --dbg("Read UTF: %s", val)
     return true, val
   end,
@@ -376,7 +374,7 @@ JavaDIS = {
     if not self.bReader:canRead(8)  then-- Length of the string is two bytes
       return false, "Not enough data in buffer [3]"
     end
-    return true, self.bReader:unpack('H8')
+    return true, stdnse.tohex(self.bReader:unpack('c8'))
 
   end,
   skip = function(self, len)
@@ -419,7 +417,7 @@ JavaClass = {
   isExternalizable = function(self)
     if self._binaryFlags == nil then return false end
 
-    return bit.band(self._binaryflags, RMIUtils.SC_EXTERNALIZABLE)
+    return (self._binaryflags & RMIUtils.SC_EXTERNALIZABLE)
   end,
 
   addField = function( self, field )
@@ -553,12 +551,11 @@ JavaField = {
   getValue = function( self ) return self.value end,
 
   __tostring = function( self )
-    local data = tostring(self.type) .. " " .. tostring(self.name)
     if self.value ~= nil then
-      data = data .." = " .. tostring(self.value)
+      return string.format("%s %s = %s", self.type, self.name, self.value)
+    else
+      return string.format("%s %s", self.type, self.name)
     end
-
-    return data
   end,
   toTable = function(self)
     local data = {tostring(self.type) .. " " .. tostring(self.name)}
@@ -571,8 +568,7 @@ JavaField = {
           table.insert(data, self.value)
         end
       else
-        --TODO: FIXME This is illegal, but I don't know what the intent was:
-        data = data .." = " .. tostring(self.value) --FIXME
+        table.insert(data, self.value)
       end
     end
     return data
@@ -721,7 +717,6 @@ function RmiDataStream:connect(host, port)
   -- Start sending a message --
   -- Add Header, Version and Protocol
 
-  --dos:write('JRMI' .. bin.pack('H', Version .. Proto.Stream))
   dos:writeInt(1246907721) -- == JRMI
   dos:writeShort(Version)
   dos:writeByte(Proto.Stream)
@@ -788,7 +783,7 @@ function RmiDataStream:writeMethodCall(out,objNum, hash, op, arguments)
   -- Send Call:
   dos:writeByte(0x50)
   -- Send Magic 0xaced
-  dos:writeShort(0xACED)
+  dos:writeUnsignedShort(0xACED)
   -- Send version 0x0005
   dos:writeShort(0x0005)
   -- Send TC_BLOKDATA
@@ -815,7 +810,7 @@ function RmiDataStream:writeMethodCall(out,objNum, hash, op, arguments)
   dos:writeLong(time);
   dos:writeShort(count);
   dos:writeInt(op)
-  dos:pack('H',hash)
+  dos:write(stdnse.fromhex(hash))
 
   -- And now, the arguments
   if arguments ~= nil then
@@ -885,7 +880,7 @@ function RmiDataStream:readReturnData(dis)
     return doh("Not enough data received")
   end
 
-  local status, magic = dis:readShort() -- read magic
+  local status, magic = dis:readUnsignedShort() -- read magic
   local status, version = dis:readShort() -- read version
 
 
@@ -909,8 +904,8 @@ function RmiDataStream:readReturnData(dis)
 
 
   if(dis.bReader:bufferSize() > 0) then
-    local content = dis.bReader:unpack('H'..tostring(dis.bReader:bufferSize()))
-    dbg("Buffer content: %s" ,content)
+    local content = dis.bReader:unpack('c'..tostring(dis.bReader:bufferSize()))
+    dbg("Buffer content: %s", stdnse.tohex(content))
   end
   return status, x
 end
@@ -1033,7 +1028,6 @@ function readExternalData(dis)
     if tc == TC.TC_BLOCKDATA then
       status, len = dis:readByte()
       status, content = dis.bReader:skip(len)
-      --print(bin.unpack("H"..tostring(#content),content))
       --print(makeStringReadable(content))
       dbg("Read external data (%d bytes): %s " ,len, content)
       --local object = ExternalClassParsers['java.rmi.server.RemoteObject'](dis)
@@ -1066,11 +1060,14 @@ ExternalClassParsers = {
   --@see sun.rmi.server.UnicastRef
   --@see sun.rmi.server.UnicastRef2
   UnicastRef = function(dis)
-    local stat, host = dis:readUTF();
-    if not stat then return doh("Parsing external data, could not read host (UTF)") end
-    local status, port = dis:readUnsignedInt();
-    if not stat then return doh("Parsing external data, could not read port (int)") end
-
+    local sts_host, host = dis:readUTF()
+    if not sts_host then
+      return doh("Parsing external data, could not read host (UTF)")
+    end
+    local sts_port, port = dis:readUnsignedInt()
+    if not sts_port then
+      return doh("Parsing external data, could not read port (int)")
+    end
     dbg("a host: %s, port %d", host, port)
     return true, ("@%s:%d"):format(host,port)
   end,
@@ -1079,24 +1076,29 @@ ExternalClassParsers = {
   --@see sun.rmi.server.UnicastRef
   --@see sun.rmi.server.UnicastRef2
   UnicastRef2 = function(dis)
-    local stat, form = dis:readByte();
-    if not stat then return doh("Parsing external data, could not read byte") end
-    if form == 0  or form == 1 then-- FORMAT_HOST_PORT or  FORMAT_HOST_PORT_FACTORY
-      local stat, host = dis:readUTF();
-      if not stat then return doh("Parsing external data, could not read host (UTF)") end
-      local status, port = dis:readUnsignedInt();
-      if not stat then return doh("Parsing external data, could not read port (int)") end
-      dbg("b host: %s, port %d", host, port)
-      if form ==0 then
-        return true, ("@%s:%d"):format(host,port)
-      end
-      -- for FORMAT_HOST_PORT_FACTORY, there's an object left to read
-      local status, object = readObject0(dis)
-      return true, ("@%s:%d"):format(host,port)
-      --return true, {host = host, port = port, factory = object}
-    else
+    local sts_form, form = dis:readByte()
+    if not sts_form then
+      return doh("Parsing external data, could not read byte")
+    end
+    if not (form == 0 or form == 1) then-- FORMAT_HOST_PORT or  FORMAT_HOST_PORT_FACTORY
       return doh("Invalid endpoint format")
     end
+    local sts_host, host = dis:readUTF()
+    if not sts_host then
+      return doh("Parsing external data, could not read host (UTF)")
+    end
+    local sts_port, port = dis:readUnsignedInt()
+    if not sts_port then
+      return doh("Parsing external data, could not read port (int)")
+    end
+    dbg("b host: %s, port %d", host, port)
+    if form == 0 then
+      return true, ("@%s:%d"):format(host,port)
+    end
+    -- for FORMAT_HOST_PORT_FACTORY, there's an object left to read
+    local sts_object, object = readObject0(dis)
+    return true, ("@%s:%d"):format(host,port)
+    --return true, {host = host, port = port, factory = object}
   end
 }
 --@see java.rmi.server.RemoteObject:readObject()
@@ -1156,16 +1158,9 @@ end
 -- returns the string with all non-printable chars
 -- coded as hex
 function makeStringReadable(data)
-  local r = ""
-  for i=1,#data,1 do
-    local x = data:byte(i)
-    if x > 31 and x <127 then
-      r = r .. data:sub(i,i)
-    else
-      r = r .. ("\\x%x"):format(x)
-    end
-  end
-  return r
+  return data:gsub("[\x00-\x1f\x7f-\xff]", function (x)
+      return ("\\x%02x"):format(x:byte())
+    end)
 end
 
 function readNonProxyDesc(dis)
@@ -1348,7 +1343,7 @@ function readFieldDesc(dis)
     -- on the field
     local status, fieldclassname = readTypeString(dis)
     if not status then return doh("Could not read data") end
-    if char == '[s' then
+    if char == '[' then
       fieldType = fieldclassname .. " []"
     else
       fieldType = fieldclassname
@@ -1519,19 +1514,19 @@ RMIUtils = {
 
   flagsToString = function(flags)
     local retval = ''
-    if ( bit.band(flags, RMIUtils.SC_WRITE_METHOD) ~= 0) then
+    if ( (flags & RMIUtils.SC_WRITE_METHOD) ~= 0) then
       retval = retval .. " WRITE_METHOD"
     end
-    if ( bit.band(flags, RMIUtils.SC_BLOCK_DATA) ~= 0) then
+    if ( (flags & RMIUtils.SC_BLOCK_DATA) ~= 0) then
       retval = retval .. " BLOCK_DATA"
     end
-    if ( bit.band(flags, RMIUtils.SC_EXTERNALIZABLE) ~= 0) then
+    if ( (flags & RMIUtils.SC_EXTERNALIZABLE) ~= 0) then
       retval = retval .. " EXTERNALIZABLE"
     end
-    if ( bit.band(flags, RMIUtils.SC_SERIALIZABLE) ~= 0) then
+    if ( (flags & RMIUtils.SC_SERIALIZABLE) ~= 0) then
       retval = retval .. " SC_SERIALIZABLE"
     end
-    if ( bit.band(flags, RMIUtils.SC_ENUM) ~= 0) then
+    if ( (flags & RMIUtils.SC_ENUM) ~= 0) then
       retval = retval .. " SC_ENUM"
     end
     return retval
